#ifndef KDP2_INF_APP_TOF_H
#define KDP2_INF_APP_TOF_H

#include <stdint.h>
#include "kp_struct.h"

#define KDP2_INF_ID_APP_TOF 500

#define YOLO_GOOD_BOX_MAX 100 /**< maximum number of bounding boxes for Yolo models */

typedef struct
{
    float x1;          /**< top-left corner: x */
    float y1;          /**< top-left corner: y */
    float x2;          /**< bottom-right corner: x */
    float y2;          /**< bottom-right corner: y */
    float score;       /**< probability score */
    int32_t class_num; /**< class # (of many) with highest probability */
    uint32_t average_depth;
} __attribute__((aligned(4))) kdp2_tof_box_t;

typedef struct
{
    uint32_t class_count;                    /**< total class count */
    uint32_t box_count;                      /**< boxes of all classes */
    kdp2_tof_box_t boxes[YOLO_GOOD_BOX_MAX]; /**< box information */
} __attribute__((aligned(4))) kdp2_app_tof_data_t;

typedef struct
{
    /* header stamp is necessary for data transfer between host and device */
    kp_inference_header_stamp_t header_stamp;
    uint8_t paddings[40];                       /**< used for align 64 bytes */
    uint8_t raw_data[];

} __attribute__((aligned(4))) kdp2_ipc_app_tof_inf_header_t;

typedef struct
{
    /* header stamp is necessary for data transfer between host and device */
    kp_inference_header_stamp_t header_stamp; // JID_HICO_TOF_RESULT
    uint8_t paddings[40];                       /**< used for align 64 bytes */
    uint8_t nir_data[640 * 480];
    uint8_t depth_data[640 * 480 * 2];
    kdp2_app_tof_data_t tof_data;

} __attribute__((aligned(4))) kdp2_app_tof_result_t;

void kdp2_app_tof_inference(int job_id, int num_input_buf, void **inf_input_buf_list);

#endif
